"""
This module is to be imported in the dynamically created and updated entity.py module.
"""

from typing import TYPE_CHECKING, Type, TypeVar, Union

from pydantic import BaseModel

T = TypeVar("T", bound=BaseModel)

# this is dirty, but required for autocompletion: https://stackoverflow.com/questions/62884543/pydantic-autocompletion-in-vs-code
# idealy solved by custom templates in the future: https://github.com/koxudaxi/datamodel-code-generator/issues/860
# ToDo: Still needed?

if TYPE_CHECKING:
    from dataclasses import dataclass as _basemodel_decorator
else:
    _basemodel_decorator = lambda x: x  # noqa: E731


@_basemodel_decorator
class OswBaseModel(BaseModel):
    def full_dict(self, **kwargs):  # extent BaseClass export function
        d = super().dict(**kwargs)
        for key in ("_osl_template", "_osl_footer"):
            if hasattr(self, key):
                d[key] = getattr(self, key)
                # Include selected private properties. note: private properties are not
                #  considered as discriminator
        return d

    def cast(self, cls: Union[Type[T], type], **kwargs) -> T:
        """Casting self into target class

        Parameters
        ----------
            cls
                target class
            kwargs
                additional attributes to be set

        Returns
        -------
        instance of target class
        """
        combined_args = {**self.dict(), **kwargs}
        del combined_args["type"]
        return cls(**combined_args)

    def cast_none_to_default(self, cls: Union[Type[T], type], **kwargs) -> T:
        """Casting self into target class. If the passed attribute is None or solely
        includes None values, the attribute is not passed to the instance of the
        target class, which will then fall back to the default."""

        def test_if_empty_list_or_none(obj) -> bool:
            if obj is None:
                return True
            elif isinstance(obj, list):
                if len(obj) == 0:
                    return True
                elif len([item for item in obj if item is not None]) == 0:
                    return True
            return False

        self_args = {
            k: v for k, v in self.dict().items() if not test_if_empty_list_or_none(v)
        }
        combined_args = {**self_args, **kwargs}
        del combined_args["type"]
        return cls(**combined_args)


class Ontology(OswBaseModel):
    iri: str
    prefix: str
    name: str
    prefix_name: str
    link: str



import os
from io import StringIO

import osw.model.entity as model
from osw.auth import CredentialManager
from osw.core import OSW
from osw.utils.wiki import get_full_title
from osw.wtsite import WtSite

# install: pip install osw[S3]

# credential manager
# can use a file or hardcode the credentials, otherwise the user will be prompted to enter them
cm = CredentialManager(
    cred_filepath=os.path.join(
        os.path.dirname(os.path.abspath(__file__)), "accounts.pwd.yaml"
    )
    # CredentialManager.UserPwdCredential(
    #     iri=wiki_domain, username=wiki_username, password=wiki_password
    # )
)
# change the domain to your osw instance
osw_obj = OSW(
    site=WtSite(WtSite.WtSiteConfig(iri="wiki-dev.open-semantic-lab.org", cred_mngr=cm))
)

# load the required schemas / data classes
if not hasattr(model, "S3File"):
    osw_obj.fetch_schema(
        OSW.FetchSchemaParam(
            schema_title=[
                "Category:OSW11a53cdfbdc24524bf8ac435cbf65d9d",  # WikiFile
                "Category:OSW3e3f5dd4f71842fbb8f270e511af8031",  # LocalFile
                "Category:OSWc43f749badcb4490a785505de1fc7d20",  # S3File
            ],
            mode="replace",
        )
    )

# import the controller modules
# note: since they depend on the data classes, they must be imported after the schemas are loaded
from osw.controller.file.local import (  # noqa (ignore flake8 warning)
    LocalFileController,
)
from osw.controller.file.s3 import S3FileController  # noqa (ignore flake8 warning)
from osw.controller.file.wiki import WikiFileController  # noqa (ignore flake8 warning)


def video_file():
    file = osw_obj.load_entity(
        "File:OSW7b2398a60f004006b9a6ef89210858f3.mp4"
    )  # the file
    wf2 = file.cast(WikiFileController, osw=osw_obj)  # the file controller
    # wf2.suffix = "mp4"
    lf2 = LocalFileController.from_other(wf2, path="dummy2.mp4")
    lf2.put_from(wf2)


def wiki_file():
    # create a local file
    # with open("dummy.txt", "w") as f:
    #    f.write("Hello World!")
    lf = LocalFileController(path="dummy.txt")  # here an uuid already exists
    lf.put(StringIO("Hello World!"))

    # create a remote file (here: a wiki file)
    wf = WikiFileController(osw=osw_obj)
    # or cast to wiki wile to keep all common attributes
    wf = lf.cast(WikiFileController, osw=osw_obj)
    # which is equivalent to
    wf = WikiFileController.from_other(lf, osw=osw_obj)

    # upload the local file to the remote file
    wf.put_from(lf)
    # write new content on the fly
    wf.put(StringIO("Some new content"))

    # get an existing file
    file = osw_obj.load_entity(f"{wf.namespace}:{wf.title}")  # the file
    wf2 = file.cast(WikiFileController, osw=osw_obj)  # the file controller
    lf2 = LocalFileController.from_other(wf2, path="dummy2.txt")
    lf2.put_from(wf2)
    get_full_title(wf2)

    # delete the files
    lf.delete()
    wf.delete()  # note: wf2 actually points to the same file as wf
    lf2.delete()


def s3_file():
    # make sure your credential file contains a access_key_id=username,
    # and secret_access_key=password for the s3 domain
    file = osw_obj.load_entity("Item:OSW5f53ed0b5c354fc3b1a122b9066744f3")  # the file
    s3f = file.cast(S3FileController, cm=cm)  # the file controller

    # download
    lf = LocalFileController.from_other(s3f, path="s3_test.txt")
    lf.put_from(s3f)

    # upload
    s3f.put_from(lf)

    s3f = model.S3File(
        url="https://s3.example10.open-semantic-lab.org/test-bucket/test-example2.txt",
        label=[model.Label(text="NewFile")],
    ).cast(S3FileController, cm=cm)
    s3f.put(StringIO("Hello World!"))

    s3f.delete()
